package com.sun.java.swing.plaf.windows;

import sun.swing.UIClientPropertyKey;
import java.awt.AlphaComposite;
import java.awt.Graphics2D;
import javax.swing.SwingUtilities;
import java.security.AccessController;
import sun.security.action.GetBooleanAction;
import java.util.Iterator;
import java.util.ArrayList;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.awt.Graphics;
import java.util.EnumMap;
import javax.swing.JTabbedPane;
import javax.swing.UIManager;
import java.util.WeakHashMap;
import sun.awt.AppContext;
import javax.swing.Timer;
import javax.swing.JComponent;
import java.util.Map;
import java.beans.PropertyChangeListener;
import java.awt.event.ActionListener;

class AnimationController implements ActionListener, PropertyChangeListener
{
    private static final boolean VISTA_ANIMATION_DISABLED;
    private static final Object ANIMATION_CONTROLLER_KEY;
    private final Map<JComponent, Map<TMSchema.Part, AnimationController.AnimationState>> animationStateMap;
    private final Timer timer;
    
    private static synchronized AnimationController getAnimationController() {
        AppContext appContext = AppContext.getAppContext();
        Object value = appContext.get(AnimationController.ANIMATION_CONTROLLER_KEY);
        if (value == null) {
            value = new AnimationController();
            appContext.put(AnimationController.ANIMATION_CONTROLLER_KEY, value);
        }
        return (AnimationController)value;
    }
    
    private AnimationController() {
        this.animationStateMap = new WeakHashMap();
        (this.timer = new Timer(33, this)).setRepeats(true);
        this.timer.setCoalesce(true);
        UIManager.addPropertyChangeListener(this);
    }
    
    private static void triggerAnimation(JComponent component, TMSchema.Part part, TMSchema.State state) {
        if (component instanceof JTabbedPane || part == TMSchema.Part.TP_BUTTON) {
            return;
        }
        AnimationController animationController = getAnimationController();
        TMSchema.State state2 = animationController.getState(component, part);
        if (state2 != state) {
            animationController.putState(component, part, state);
            if (state == TMSchema.State.DEFAULTED) {
                state2 = TMSchema.State.HOT;
            }
            if (state2 != null) {
                long n;
                if (state == TMSchema.State.DEFAULTED) {
                    n = 1000L;
                }
                else {
                    XPStyle xp = XPStyle.getXP();
                    n = ((xp != null) ? xp.getThemeTransitionDuration(component, part, normalizeState(state2), normalizeState(state), TMSchema.Prop.TRANSITIONDURATIONS) : 1000L);
                }
                animationController.startAnimation(component, part, state2, state, n);
            }
        }
        return;
    }
    
    private static TMSchema.State normalizeState(TMSchema.State state) {
        TMSchema.State state2;
        switch (state) {
            case DOWNPRESSED:
            case LEFTPRESSED:
            case RIGHTPRESSED: {
                state2 = TMSchema.State.UPPRESSED;
                break;
            }
            case DOWNDISABLED:
            case LEFTDISABLED:
            case RIGHTDISABLED: {
                state2 = TMSchema.State.UPDISABLED;
                break;
            }
            case DOWNHOT:
            case LEFTHOT:
            case RIGHTHOT: {
                state2 = TMSchema.State.UPHOT;
                break;
            }
            case DOWNNORMAL:
            case LEFTNORMAL:
            case RIGHTNORMAL: {
                state2 = TMSchema.State.UPNORMAL;
                break;
            }
            default: {
                state2 = state;
                break;
            }
        }
        return state2;
    }
    
    private synchronized TMSchema.State getState(JComponent component, TMSchema.Part part) {
        TMSchema.State state = null;
        Object clientProperty = component.getClientProperty(AnimationController.PartUIClientPropertyKey.getKey(part));
        if ((clientProperty instanceof TMSchema.State)) {
            state = (TMSchema.State)clientProperty;
            return state;
        }
        return state;
    }
    
    private synchronized void putState(JComponent component, TMSchema.Part part, TMSchema.State value) {
        component.putClientProperty(AnimationController.PartUIClientPropertyKey.getKey(part), value);
    }
    
    private synchronized void startAnimation(JComponent component, TMSchema.Part part, TMSchema.State state, TMSchema.State state2, long n) {
        boolean n2 = false;
        if (state2 == TMSchema.State.DEFAULTED) {
            n2 = true;
        }
        Map map = (Map<TMSchema.Part, AnimationController.AnimationState>)this.animationStateMap.get(component);
        if (n <= 0L) {
            if (map != null) {
                map.remove(part);
                if (map.size() == 0) {
                    this.animationStateMap.remove(component);
                }
            }
            return;
        }
        else {
            if (map == null) {
                map = new EnumMap(TMSchema.Part.class);
                this.animationStateMap.put(component, map);
            }
            map.put(part, new AnimationController.AnimationState(state, n, (boolean)n2));
            if (!this.timer.isRunning()) {
                this.timer.start();
            }
            return;
        }
    }
    
    static void paintSkin(JComponent component, XPStyle.Skin skin, Graphics graphics, int n, int n2, int n3, int n4, TMSchema.State state) {
        if (AnimationController.VISTA_ANIMATION_DISABLED) {
            skin.paintSkinRaw(graphics, n, n2, n3, n4, state);
            return;
        }
        triggerAnimation(component, skin.part, state);
        AnimationController animationController = getAnimationController();
        synchronized (animationController) {
            AnimationController.AnimationState animationState = null;
            Map<Object, Object> map = (Map)animationController.animationStateMap.get(component);
            if (map != null) {
                animationState = (AnimationController.AnimationState)map.get(skin.part);
            }
            if (animationState != null) {
                animationState.paintSkin(skin, graphics, n, n2, n3, n4, state);
            }
            else {
                skin.paintSkinRaw(graphics, n, n2, n3, n4, state);
            }
        }
        return;
    }
    
    @Override
    public synchronized void propertyChange(PropertyChangeEvent propertyChangeEvent) {
        if ("lookAndFeel" == propertyChangeEvent.getPropertyName() && !(propertyChangeEvent.getNewValue() instanceof WindowsLookAndFeel)) {
            this.dispose();
        }
    }
    
    @Override
    public synchronized void actionPerformed(ActionEvent actionEvent) {
        Object o = null;
        Object o2 = null;
        for (JComponent component : this.animationStateMap.keySet()) {
            component.repaint();
            if (o2 != null) {
                o2.clear();
            }
            Map<Object, Object> map = (Map)this.animationStateMap.get(component);
            if (!component.isShowing() || map == null || map.size() == 0) {
                if (o == null) {
                    o = new ArrayList();
                }
                o.add(component);
            }
            else {
                for (TMSchema.Part part : map.keySet()) {
                    if (((AnimationController.AnimationState)map.get(part)).isDone()) {
                        if (o2 == null) {
                            o2 = new ArrayList();
                        }
                        o2.add(part);
                    }
                    else {
                        continue;
                    }
                }
                if (o2 != null) {
                    if (o2.size() == map.size()) {
                        if (o == null) {
                            o = new ArrayList();
                        }
                        o.add(component);
                    }
                    else {
                        Iterator<Object> iterator3 = o2.iterator();
                        while (iterator3.hasNext()) {
                            map.remove((Object)iterator3.next());
                        }
                    }
                }
                else {
                    continue;
                }
            }
        }
        if (o != null) {
            Iterator<Object> iterator4 = o.iterator();
            while (iterator4.hasNext()) {
                this.animationStateMap.remove((Object)iterator4.next());
            }
        }
        if (this.animationStateMap.size() == 0) {
            this.timer.stop();
        }
    }
    
    private synchronized void dispose() {
        this.timer.stop();
        UIManager.removePropertyChangeListener(this);
        synchronized (AnimationController.class) {
            AppContext.getAppContext().put(AnimationController.ANIMATION_CONTROLLER_KEY, null);
        }
    }
    
    static {
        VISTA_ANIMATION_DISABLED = (boolean)AccessController.doPrivileged(new GetBooleanAction("swing.disablevistaanimation"));
        ANIMATION_CONTROLLER_KEY = new StringBuilder("ANIMATION_CONTROLLER_KEY");
    }
    
    private static class AnimationState
    {
        private final TMSchema.State startState;
        private final long duration;
        private long startTime;
        private boolean isForward;
        private boolean isForwardAndReverse;
        private float progress;
        
        AnimationState(TMSchema.State startState, long n, boolean isForwardAndReverse) {
            this.isForward = true;
            assert startState != null && n > 0L;
            assert !(!SwingUtilities.isEventDispatchThread());
            this.startState = startState;
            this.duration = n * 1000000L;
            this.startTime = System.nanoTime();
            this.isForwardAndReverse = isForwardAndReverse;
            this.progress = 0.0f;
            return;
        }
        
        private void updateProgress() {
            assert !(!SwingUtilities.isEventDispatchThread());
            if (this.isDone()) {
                return;
            }
            long nanoTime = System.nanoTime();
            this.progress = (float)(nanoTime - this.startTime) / (float)this.duration;
            this.progress = Math.max(this.progress, 0.0f);
            if (this.progress >= 1.0f) {
                this.progress = 1.0f;
                if (this.isForwardAndReverse) {
                    this.startTime = nanoTime;
                    this.progress = 0.0f;
                    this.isForward = !this.isForward;
                }
            }
            return;
        }
        
        void paintSkin(XPStyle.Skin skin, Graphics graphics, int n, int n2, int n3, int n4, TMSchema.State state) {
            assert !(!SwingUtilities.isEventDispatchThread());
            this.updateProgress();
            if (!this.isDone()) {
                Graphics2D graphics2D = (Graphics2D)graphics.create();
                skin.paintSkinRaw(graphics2D, n, n2, n3, n4, this.startState);
                float progress;
                if (this.isForward) {
                    progress = this.progress;
                }
                else {
                    progress = 1.0f - this.progress;
                }
                graphics2D.setComposite(AlphaComposite.SrcOver.derive(progress));
                skin.paintSkinRaw(graphics2D, n, n2, n3, n4, state);
                graphics2D.dispose();
            }
            else {
                skin.paintSkinRaw(graphics, n, n2, n3, n4, state);
            }
            return;
        }
        
        boolean isDone() {
            assert !(!SwingUtilities.isEventDispatchThread());
            return this.progress >= 1.0f;
        }
    }
    
    private static class PartUIClientPropertyKey implements UIClientPropertyKey
    {
        private static final Map<TMSchema.Part, AnimationController.PartUIClientPropertyKey> map;
        private final TMSchema.Part part;
        
        static synchronized AnimationController.PartUIClientPropertyKey getKey(TMSchema.Part part) {
            AnimationController.PartUIClientPropertyKey partUIClientPropertyKey = (AnimationController.PartUIClientPropertyKey)AnimationController.PartUIClientPropertyKey.map.get(part);
            if (partUIClientPropertyKey == null) {
                partUIClientPropertyKey = new PartUIClientPropertyKey(part);
                AnimationController.PartUIClientPropertyKey.map.put(part, partUIClientPropertyKey);
                return partUIClientPropertyKey;
            }
            return partUIClientPropertyKey;
        }
        
        private PartUIClientPropertyKey(TMSchema.Part part) {
            this.part = part;
        }
        
        @Override
        public String toString() {
            return this.part.toString();
        }
        
        static {
            map = new EnumMap(TMSchema.Part.class);
        }
    }
}
